@@ -38,11 +38,6 @@ pyg_destroy_notify(gpointer user_data)
 
 /* -------------- GParamSpec objects ---------------------------- */
 
-typedef struct {
-    PyObject_HEAD
-    GParamSpec *pspec;
-} PyGParamSpec;
-
 static int
 pyg_param_spec_compare(PyGParamSpec *self, PyGParamSpec *v)
 {
@@ -138,7 +133,7 @@ PyTypeObject PyGParamSpec_Type = {
     NULL
 };
 
-static PyObject *
+PyObject *
 pyg_param_spec_new(GParamSpec *pspec)
 {
     PyGParamSpec *self;
@@ -390,7 +385,7 @@ pyg_object_set_property (GObject *object, guint property_id,
     g_return_if_fail(object_wrapper != NULL);
 
     py_pspec = pyg_param_spec_new(pspec);
-    py_value = pyg_value_as_pyobject (value);
+    py_value = pyg_value_as_pyobject (value, TRUE);
     retval = PyObject_CallMethod(object_wrapper, "do_set_property",
 				 "OO", py_pspec, py_value);
     if (retval) {
@@ -1284,6 +1279,7 @@ pyg_error_check(GError **error)
 	}
 
 	PyErr_SetObject(gerror_exc, exc_instance);
+	Py_DECREF(exc_instance);
 	g_clear_error(error);
 	return TRUE;
     }
@@ -1299,6 +1295,7 @@ struct _PyGObject_Functions pygobject_api_functions = {
   pygobject_new,
 
   pyg_closure_new,
+  pygobject_watch_closure,
   pyg_destroy_notify,
 
   pyg_type_from_object,
