@@ -17,19 +17,6 @@ PyGdkWindow_New(GdkWindow *win)
     gdk_window_ref(self->obj);
     return (PyObject *)self;
 }
-
-PyObject *
-PyGdkGC_New(GdkGC *gc)
-{
-    PyGdkGC_Object *self;
-
-    self = (PyGdkGC_Object *)PyObject_NEW(PyGdkGC_Object, &PyGdkGC_Type);
-    if (self == NULL)
-	return NULL;
-    self->obj = gc;
-    gdk_gc_ref(self->obj);
-    return (PyObject *)self;
-}
 #endif
 
 PyObject *
@@ -693,204 +680,6 @@ PyTypeObject PyGdkWindow_Type = {
     0L,0L,0L,0L,
     NULL
 };
-
-static void
-PyGdkGC_Dealloc(PyGdkGC_Object *self)
-{
-    gdk_gc_unref(self->obj);
-    PyObject_DEL(self);
-}
-
-static PyObject *
-PyGdkGC_set_dashes(PyGdkGC_Object *self, PyObject *args)
-{
-    gint dash_offset, n, i;
-    PyObject *list;
-    guchar *dash_list;
-
-    if (!PyArg_ParseTuple(args, "iO:GdkGC.set_dashes", &dash_offset, &list))
-	return NULL;
-    if (!PySequence_Check(list)) {
-	PyErr_SetString(PyExc_TypeError, "second argument must be a sequence");
-	return NULL;
-    }
-    n = PySequence_Length(list);
-    dash_list = g_new(char, n);
-    for (i = 0; i < n; i++) {
-	PyObject *item = PySequence_GetItem(list, i);
-	Py_DECREF(item);
-
-	if (!PyInt_Check(item)) {
-	    PyErr_SetString(PyExc_TypeError, "sequence member must be an int");
-	    g_free(dash_list);
-	    return NULL;
-	}
-	dash_list[i] = (guchar)PyInt_AsLong(item);
-	if (dash_list[i] == 0) {
-	    PyErr_SetString(PyExc_TypeError, "sequence member must not be 0");
-	    g_free(dash_list);
-	    return NULL;
-	}
-    }
-    gdk_gc_set_dashes(self->obj, dash_offset, dash_list, n);
-    g_free(dash_list);
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
-static PyMethodDef PyGdkGC_methods[] = {
-    {"set_dashes", (PyCFunction)PyGdkGC_set_dashes, METH_VARARGS, NULL},
-    {NULL, 0, 0, NULL}
-};
-
-static PyObject *
-PyGdkGC_GetAttr(PyGdkGC_Object *self, char *key)
-{
-    GdkGCValues gc;
-
-    if (!strcmp(key, "__members__"))
-	return Py_BuildValue("[ssssssssssssssssss]", "background", "cap_style",
-			     "clip_mask", "clip_x_origin", "clip_y_origin",
-			     "fill", "font", "foreground", "function",
-			     "graphics_exposures", "join_style", "line_style",
-			     "line_width", "stipple", "sub_window", "tile",
-			     "ts_x_origin", "ts_y_origin");
-    gdk_gc_get_values(self->obj, &gc);
-    if (!strcmp(key, "foreground")) return PyGdkColor_New(&(gc.foreground));
-    if (!strcmp(key, "background")) return PyGdkColor_New(&(gc.background));
-    if (!strcmp(key, "font")) return PyGdkFont_New(gc.font);
-    if (!strcmp(key, "function")) return PyInt_FromLong(gc.function);
-    if (!strcmp(key, "fill")) return PyInt_FromLong(gc.fill);
-    if (!strcmp(key, "tile")) {
-	if (gc.tile != NULL) return PyGdkWindow_New(gc.tile);
-	Py_INCREF(Py_None); return Py_None;
-    }
-    if (!strcmp(key, "stipple")) {
-	if (gc.stipple != NULL) return PyGdkWindow_New(gc.stipple);
-	Py_INCREF(Py_None); return Py_None;
-    }
-    if (!strcmp(key, "clip_mask")) {
-	if (gc.clip_mask!=NULL) return PyGdkWindow_New(gc.clip_mask);
-	Py_INCREF(Py_None); return Py_None;
-    }
-    if (!strcmp(key, "subwindow_mode"))
-	return PyInt_FromLong(gc.subwindow_mode);
-    if (!strcmp(key, "ts_x_origin")) return PyInt_FromLong(gc.ts_x_origin);
-    if (!strcmp(key, "ts_y_origin")) return PyInt_FromLong(gc.ts_y_origin);
-    if (!strcmp(key, "clip_x_origin")) return PyInt_FromLong(gc.clip_x_origin);
-    if (!strcmp(key, "clip_y_origin")) return PyInt_FromLong(gc.clip_y_origin);
-    if (!strcmp(key, "graphics_exposures"))
-	return PyInt_FromLong(gc.graphics_exposures);
-    if (!strcmp(key, "line_width")) return PyInt_FromLong(gc.line_width);
-    if (!strcmp(key, "line_style")) return PyInt_FromLong(gc.line_style);
-    if (!strcmp(key, "cap_style")) return PyInt_FromLong(gc.cap_style);
-    if (!strcmp(key, "join_style")) return PyInt_FromLong(gc.join_style);
-
-    return Py_FindMethod(PyGdkGC_methods, (PyObject *)self, key);
-}
-
-static int
-PyGdkGC_SetAttr(PyGdkGC_Object *self, char *key, PyObject *value)
-{
-    GdkGC *gc = self->obj;
-
-    if (value == NULL) {
-	PyErr_SetString(PyExc_TypeError, "can't delete attributes");
-	return -1;
-    }
-    if (PyInt_Check(value)) {
-	int i = PyInt_AsLong(value);
-	GdkGCValues v;
-	gdk_gc_get_values(gc, &v);
-	if (!strcmp(key, "function")) gdk_gc_set_function(gc, i);
-	else if (!strcmp(key, "fill")) gdk_gc_set_fill(gc, i);
-	else if (!strcmp(key, "subwindow_mode")) gdk_gc_set_subwindow(gc, i);
-	else if (!strcmp(key, "ts_x_origin"))
-	    gdk_gc_set_ts_origin(gc, i, v.ts_y_origin);
-	else if (!strcmp(key, "ts_y_origin"))
-	    gdk_gc_set_ts_origin(gc, v.ts_x_origin, i);
-	else if (!strcmp(key, "clip_x_origin"))
-	    gdk_gc_set_clip_origin(gc, i, v.clip_y_origin);
-	else if (!strcmp(key, "clip_y_origin"))
-	    gdk_gc_set_clip_origin(gc, v.clip_x_origin, i);
-	else if (!strcmp(key, "graphics_exposures")) gdk_gc_set_exposures(gc, i);
-	else if (!strcmp(key, "line_width"))
-	    gdk_gc_set_line_attributes(gc, i,v.line_style,v.cap_style,v.join_style);
-	else if (!strcmp(key, "line_style"))
-	    gdk_gc_set_line_attributes(gc, v.line_width,i,v.cap_style,v.join_style);
-	else if (!strcmp(key, "cap_style"))
-	    gdk_gc_set_line_attributes(gc, v.line_width,v.line_style,i,v.join_style);
-	else if (!strcmp(key, "join_style"))
-	    gdk_gc_set_line_attributes(gc, v.line_width,v.line_style,v.cap_style,i);
-	else {
-	    PyErr_SetString(PyExc_TypeError,"type mismatch");
-	    return -1;
-	}
-    } else if (PyGdkColor_Check(value)) {
-	GdkColor *c = PyGdkColor_Get(value);
-	if (!strcmp(key, "foreground")) gdk_gc_set_foreground(gc, c);
-	else if (!strcmp(key, "background")) gdk_gc_set_background(gc, c);
-	else {
-	    PyErr_SetString(PyExc_TypeError,"type mismatch");
-	    return -1;
-	}
-    } else if (PyGdkFont_Check(value)) {
-	if (!strcmp(key, "font")) gdk_gc_set_font(gc, PyGdkFont_Get(value));
-	else{
-	    PyErr_SetString(PyExc_TypeError,"type mismatch");
-	    return -1;
-	}
-    } else if (PyGdkWindow_Check(value) || value == Py_None) {
-	GdkWindow *w = (value==Py_None)?NULL:PyGdkWindow_Get(value);
-	if (!strcmp(key, "tile")) gdk_gc_set_tile(gc, w);
-	else if (!strcmp(key, "stipple")) gdk_gc_set_stipple(gc, w);
-	else if (!strcmp(key, "clip_mask")) gdk_gc_set_clip_mask(gc, w);
-	else {
-	    PyErr_SetString(PyExc_TypeError,"type mismatch");
-	    return -1;
-	}
-    } else {
-	PyErr_SetString(PyExc_TypeError,"type mismatch");
-	return -1;
-    }
-    return 0;
-}
-
-static int
-PyGdkGC_Compare(PyGdkGC_Object *self, PyGdkGC_Object *v)
-{
-    if (self->obj == v->obj) return 0;
-    if (self->obj > v->obj) return -1;
-    return 1;
-}
-
-static long
-PyGdkGC_Hash(PyGdkGC_Object *self)
-{
-    return (long)self->obj;
-}
-
-PyTypeObject PyGdkGC_Type = {
-    PyObject_HEAD_INIT(NULL)
-    0,
-    "GdkGC",
-    sizeof(PyGdkGC_Object),
-    0,
-    (destructor)PyGdkGC_Dealloc,
-    (printfunc)0,
-    (getattrfunc)PyGdkGC_GetAttr,
-    (setattrfunc)PyGdkGC_SetAttr,
-    (cmpfunc)PyGdkGC_Compare,
-    (reprfunc)0,
-    0,
-    0,
-    0,
-    (hashfunc)PyGdkGC_Hash,
-    (ternaryfunc)0,
-    (reprfunc)0,
-    0L,0L,0L,0L,
-    NULL
-};
 #endif
 
 GdkAtom
@@ -985,7 +774,13 @@ pygtk_tree_path_to_pyobject(GtkTreePath *path)
 GtkTreePath *
 pygtk_tree_path_from_pyobject(PyObject *object)
 {
-    if (PyTuple_Check(object)) {
+    if (PyInt_Check(object)) {
+	GtkTreePath *path;
+
+	path = gtk_tree_path_new();
+	gtk_tree_path_append_index(path, PyInt_AsLong(object));
+	return path;
+    } else if (PyTuple_Check(object)) {
 	GtkTreePath *path;
 	guint len, i;
 
@@ -1030,6 +825,43 @@ PyGtkTreePath_to_value(GValue *value, PyObject *object)
     return -1;
 }
 
+gboolean
+pygdk_rectangle_from_pyobject(PyObject *object, GdkRectangle *rectangle)
+{
+    g_return_val_if_fail(rectangle != NULL, FALSE);
+
+    if (pyg_boxed_check(object, GDK_TYPE_RECTANGLE)) {
+	*rectangle = *pyg_boxed_get(object, GdkRectangle);
+	return TRUE;
+    }
+    if (PyArg_ParseTuple(object, "iiii", &rectangle->x, &rectangle->y,
+				&rectangle->width, &rectangle->height)) {
+	return TRUE;
+    }
+    PyErr_Clear();
+    PyErr_SetString(PyExc_TypeError, "could not convert to GdkRectangle");
+    return FALSE;
+}
+
+static PyObject *
+PyGdkRectangle_from_value(const GValue *value)
+{
+    GdkRectangle *rect = (GtkTreePath *)g_value_get_boxed(value);
+
+    return pyg_boxed_new(rect, GDK_TYPE_RECTANGLE, TRUE, TRUE);
+}
+static int
+PyGdkRectangle_to_value(GValue *value, PyObject *object)
+{
+    GdkRectangle rect;
+
+    if (!pygdk_rectangle_from_pyobject(object, &rect))
+	return -1;
+
+    g_value_set_boxed(value, &rect);
+    return 0;
+}
+
 /* We have to set ob_type here because stupid win32 does not allow you
  * to use variables from another dll in a global variable initialisation.
  */
@@ -1043,10 +875,13 @@ _pygtk_register_boxed_types(PyObject *moddict)
     PyGdkAtom_Type.ob_type = &PyType_Type;
 #if 0
     register_tp(GdkWindow);
-    register_tp(GdkGC);
 #endif
     register_tp(GdkAtom);
+
     pyg_register_boxed_custom(GTK_TYPE_TREE_PATH,
 			      PyGtkTreePath_from_value,
 			      PyGtkTreePath_to_value);
+    pyg_register_boxed_custom(GDK_TYPE_RECTANGLE,
+			      PyGdkRectangle_from_value,
+			      PyGdkRectangle_to_value);
 }
