@@ -11,35 +11,52 @@
 /* define this to print out debug messages */
 #undef DEBUG_TREE_MODEL
 
-static void pygtk_tree_model_class_init(PyGtkTreeModelClass *klass);
-static void pygtk_tree_model_init(PyGtkTreeModel *self);
-static void pygtk_tree_model_iface_init(GtkTreeModelIface *iface);
+#ifndef _
+#  define _(s) (s)
+#endif
+
+enum {
+    PROP_LEAK_REFERENCES = 1
+};
+
+static void pygtk_generic_tree_model_class_init(PyGtkGenericTreeModelClass *klass);
+static void pygtk_generic_tree_model_init(PyGtkGenericTreeModel *self);
+static void pygtk_generic_tree_model_iface_init(GtkTreeModelIface *iface);
+static void pygtk_generic_tree_model_set_property (GObject *object,
+						   guint property_id,
+						   const GValue *value,
+						   GParamSpec *pspec);
+static void pygtk_generic_tree_model_get_property (GObject *object,
+						   guint property_id,
+						   GValue *value,
+						   GParamSpec *pspec);
+
 
 GType
-pygtk_tree_model_get_type(void)
+pygtk_generic_tree_model_get_type(void)
 {
     static GType object_type = 0;
 
     if (!object_type) {
 	static const GTypeInfo object_info = {
-	    sizeof(PyGtkTreeModelClass),
+	    sizeof(PyGtkGenericTreeModelClass),
 	    (GBaseInitFunc) NULL,
 	    (GBaseFinalizeFunc) NULL,
-	    (GClassInitFunc) pygtk_tree_model_class_init,
+	    (GClassInitFunc) pygtk_generic_tree_model_class_init,
 	    NULL, /* class_finalize */
 	    NULL, /* class_data */
-	    sizeof(PyGtkTreeModel),
+	    sizeof(PyGtkGenericTreeModel),
 	    0, /* n_preallocs */
-	    (GInstanceInitFunc) pygtk_tree_model_init,
+	    (GInstanceInitFunc) pygtk_generic_tree_model_init,
 	};
 	static const GInterfaceInfo tree_model_info = {
-	    (GInterfaceInitFunc) pygtk_tree_model_iface_init,
+	    (GInterfaceInitFunc) pygtk_generic_tree_model_iface_init,
 	    NULL,
 	    NULL,
 	};
 
 	object_type = g_type_register_static(G_TYPE_OBJECT,
-					     "PyGtkTreeModel",
+					     "PyGtkGenericTreeModel",
 					     &object_info, 0);
 	g_type_add_interface_static(object_type,
 				    GTK_TYPE_TREE_MODEL,
@@ -49,69 +66,116 @@ pygtk_tree_model_get_type(void)
 }
 
 static void
-pygtk_tree_model_class_init(PyGtkTreeModelClass *klass)
+pygtk_generic_tree_model_class_init(PyGtkGenericTreeModelClass *klass)
 {
+    GObjectClass *object_class = (GObjectClass*) klass;
+
+    object_class->get_property = pygtk_generic_tree_model_get_property;
+    object_class->set_property = pygtk_generic_tree_model_set_property;
+ 
+    g_object_class_install_property (object_class,
+				     PROP_LEAK_REFERENCES,
+				     g_param_spec_boolean ("leak_references",
+					_("Leak references"),
+					_("Enable referencing iterator "
+	"objects (this will cause a memory leak or at least a reference "
+	"counting leak). You might need it though, if you return newly "
+	"created objects."),
+					TRUE,
+					G_PARAM_READWRITE));
 }
 
-static guint pygtk_tree_model_get_flags(GtkTreeModel *tree_model);
-static gint pygtk_tree_model_get_n_columns(GtkTreeModel *tree_model);
-static GType pygtk_tree_model_get_column_type(GtkTreeModel *tree_model,
+static guint pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model);
+static gint pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model);
+static GType pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model,
 					      gint index);
-static gboolean pygtk_tree_model_get_iter(GtkTreeModel *tree_model,
+static gboolean pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 					  GtkTreeIter *iter,
 					  GtkTreePath *path);
-static GtkTreePath *pygtk_tree_model_get_path(GtkTreeModel *tree_model,
+static GtkTreePath *pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model,
 					      GtkTreeIter *iter);
-static void pygtk_tree_model_get_value(GtkTreeModel*tree_model,
+static void pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model,
 				       GtkTreeIter *iter,
 				       gint column, GValue *value);
-static gboolean pygtk_tree_model_iter_next(GtkTreeModel *tree_model,
+static gboolean pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model,
 					   GtkTreeIter *iter);
-static gboolean pygtk_tree_model_iter_children(GtkTreeModel *tree_model,
+static gboolean pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model,
 					       GtkTreeIter *iter,
 					       GtkTreeIter *parent);
-static gboolean pygtk_tree_model_iter_has_child(GtkTreeModel *tree_model,
+static gboolean pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model,
 						GtkTreeIter *iter);
-static gint pygtk_tree_model_iter_n_children(GtkTreeModel *tree_model,
+static gint pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model,
 					     GtkTreeIter *iter);
-static gboolean pygtk_tree_model_iter_nth_child(GtkTreeModel *tree_model,
+static gboolean pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model,
 						GtkTreeIter  *iter,
 						GtkTreeIter  *parent,
 						gint n);
-static gboolean pygtk_tree_model_iter_parent(GtkTreeModel *tree_model,
+static gboolean pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model,
 					     GtkTreeIter *iter,
 					     GtkTreeIter *child);
 
 static void
-pygtk_tree_model_iface_init(GtkTreeModelIface *iface)
+pygtk_generic_tree_model_iface_init(GtkTreeModelIface *iface)
 {
-  iface->get_flags = pygtk_tree_model_get_flags;
-  iface->get_n_columns = pygtk_tree_model_get_n_columns;
-  iface->get_column_type = pygtk_tree_model_get_column_type;
-  iface->get_iter = pygtk_tree_model_get_iter;
-  iface->get_path = pygtk_tree_model_get_path;
-  iface->get_value = pygtk_tree_model_get_value;
-  iface->iter_next = pygtk_tree_model_iter_next;
-  iface->iter_children = pygtk_tree_model_iter_children;
-  iface->iter_has_child = pygtk_tree_model_iter_has_child;
-  iface->iter_n_children = pygtk_tree_model_iter_n_children;
-  iface->iter_nth_child = pygtk_tree_model_iter_nth_child;
-  iface->iter_parent = pygtk_tree_model_iter_parent;
+  iface->get_flags = pygtk_generic_tree_model_get_flags;
+  iface->get_n_columns = pygtk_generic_tree_model_get_n_columns;
+  iface->get_column_type = pygtk_generic_tree_model_get_column_type;
+  iface->get_iter = pygtk_generic_tree_model_get_iter;
+  iface->get_path = pygtk_generic_tree_model_get_path;
+  iface->get_value = pygtk_generic_tree_model_get_value;
+  iface->iter_next = pygtk_generic_tree_model_iter_next;
+  iface->iter_children = pygtk_generic_tree_model_iter_children;
+  iface->iter_has_child = pygtk_generic_tree_model_iter_has_child;
+  iface->iter_n_children = pygtk_generic_tree_model_iter_n_children;
+  iface->iter_nth_child = pygtk_generic_tree_model_iter_nth_child;
+  iface->iter_parent = pygtk_generic_tree_model_iter_parent;
 }
 
 static void
-pygtk_tree_model_init(PyGtkTreeModel *self)
+pygtk_generic_tree_model_init(PyGtkGenericTreeModel *self)
 {
+    self->leak_references = TRUE;
 }
 
-PyGtkTreeModel *
-pygtk_tree_model_new(void)
+static void
+pygtk_generic_tree_model_set_property (GObject *object, guint property_id,
+				       const GValue *value, GParamSpec *pspec)
 {
-    return PYGTK_TREE_MODEL(g_object_new(PYGTK_TYPE_TREE_MODEL, NULL));
+    switch (property_id) {
+    case PROP_LEAK_REFERENCES:
+	PYGTK_GENERIC_TREE_MODEL (object)->leak_references = g_value_get_boolean (value);
+	break;
+    default:
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	break;
+    }
+ }
+ 
+static void
+pygtk_generic_tree_model_get_property (GObject *object, guint property_id,
+				       GValue *value, GParamSpec *pspec)
+{
+    switch (property_id) {
+    case PROP_LEAK_REFERENCES:
+	g_value_set_boolean (value,
+			    PYGTK_GENERIC_TREE_MODEL (object)->leak_references);
+	break;
+    default:
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	break;
+    }
 }
 
 
-/* format of GtkTreeIter's for PyGtkTreeModel:
+PyGtkGenericTreeModel *
+pygtk_generic_tree_model_new(void)
+{
+    return PYGTK_GENERIC_TREE_MODEL(
+	g_object_new(PYGTK_TYPE_GENERIC_TREE_MODEL, NULL));
+}
+
+
+/* format of GtkTreeIter's for PyGtkGenericTreeModel:
  *  user_data == python object
  *  user_data2 == floating reference?
  *
@@ -122,11 +186,11 @@ pygtk_tree_model_new(void)
 #define METHOD_PREFIX "on_"
 
 static guint
-pygtk_tree_model_get_flags(GtkTreeModel *tree_model)
+pygtk_generic_tree_model_get_flags(GtkTreeModel *tree_model)
 {
     PyObject *self, *py_ret;
 
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), 0);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -147,12 +211,12 @@ pygtk_tree_model_get_flags(GtkTreeModel *tree_model)
 }
 
 static gint
-pygtk_tree_model_get_n_columns(GtkTreeModel *tree_model)
+pygtk_generic_tree_model_get_n_columns(GtkTreeModel *tree_model)
 {
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, 0);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), 0);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), 0);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -173,12 +237,12 @@ pygtk_tree_model_get_n_columns(GtkTreeModel *tree_model)
 }
 
 static GType
-pygtk_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
+pygtk_generic_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 {
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, G_TYPE_INVALID);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), G_TYPE_INVALID);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), G_TYPE_INVALID);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
@@ -200,13 +264,13 @@ pygtk_tree_model_get_column_type(GtkTreeModel *tree_model, gint index)
 }
 
 static gboolean
-pygtk_tree_model_get_iter(GtkTreeModel *tree_model,
+pygtk_generic_tree_model_get_iter(GtkTreeModel *tree_model,
 			  GtkTreeIter *iter, GtkTreePath *path)
 {
     PyObject *self, *py_path, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), FALSE);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     g_return_val_if_fail(path != NULL, FALSE);
     /* this call finds the wrapper for this GObject */
@@ -223,6 +287,9 @@ pygtk_tree_model_get_iter(GtkTreeModel *tree_model,
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    iter->user_data = py_ret;
+	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
+		Py_DECREF((PyObject *)iter->user_data);
+	    }
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
@@ -238,12 +305,12 @@ pygtk_tree_model_get_iter(GtkTreeModel *tree_model,
 }
 
 static GtkTreePath *
-pygtk_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
+pygtk_generic_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, NULL);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), NULL);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), NULL);
     g_return_val_if_fail(iter != NULL, NULL);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -256,6 +323,9 @@ pygtk_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
     if (py_ret) {
 	GtkTreePath *path = pygtk_tree_path_from_pyobject(py_ret);
 
+	if (!path)
+	    g_warning("could not convert return value of get_tree_path() to "
+		      "a GtkTreePath");
 	Py_DECREF(py_ret);
 	return path;
     } else {
@@ -266,22 +336,23 @@ pygtk_tree_model_get_path(GtkTreeModel *tree_model, GtkTreeIter *iter)
 }
 
 static void
-pygtk_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
+pygtk_generic_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
 			   gint column, GValue *value)
 {
     PyObject *self, *py_value;
 
     g_return_if_fail(tree_model != NULL);
-    g_return_if_fail(PYGTK_IS_TREE_MODEL(tree_model));
+    g_return_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model));
     g_return_if_fail(iter != NULL);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
 
 #ifdef DEBUG_TREE_MODEL
     g_message("get_value(%p, %d)", iter, column);
+    _PyObject_Dump (iter->user_data);
 #endif
     /* init value to column type */
-    g_value_init(value, pygtk_tree_model_get_column_type(tree_model, column));
+    g_value_init(value, pygtk_generic_tree_model_get_column_type(tree_model, column));
 
     py_value = PyObject_CallMethod(self, METHOD_PREFIX "get_value",
 				   "(Oi)", (PyObject *)iter->user_data,column);
@@ -296,12 +367,12 @@ pygtk_tree_model_get_value(GtkTreeModel*tree_model, GtkTreeIter *iter,
 }
 
 static gboolean
-pygtk_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
+pygtk_generic_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), FALSE);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -315,6 +386,9 @@ pygtk_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
 	    iter->user_data = py_ret;
+	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
+		Py_DECREF((PyObject *)iter->user_data);
+	    }
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
@@ -330,13 +404,13 @@ pygtk_tree_model_iter_next(GtkTreeModel *tree_model, GtkTreeIter *iter)
 }
 
 static gboolean
-pygtk_tree_model_iter_children(GtkTreeModel *tree_model, GtkTreeIter *iter,
+pygtk_generic_tree_model_iter_children(GtkTreeModel *tree_model, GtkTreeIter *iter,
 			       GtkTreeIter *parent)
 {
     PyObject *self, *py_ret, *py_parent = Py_None;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), FALSE);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -344,13 +418,17 @@ pygtk_tree_model_iter_children(GtkTreeModel *tree_model, GtkTreeIter *iter,
 #ifdef DEBUG_TREE_MODEL
     g_message("iter_children(%p, %p)", iter, parent);
 #endif
-    if (parent) py_parent = (PyObject *)parent->user_data;
+    if (parent && parent->user_data != NULL)
+	py_parent = (PyObject *)parent->user_data;
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_children",
 				 "(O)", py_parent);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
 	    iter->user_data = py_ret;
+	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
+		Py_DECREF((PyObject *)iter->user_data);
+	    }
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
@@ -366,12 +444,12 @@ pygtk_tree_model_iter_children(GtkTreeModel *tree_model, GtkTreeIter *iter,
 }
 
 static gboolean
-pygtk_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *iter)
+pygtk_generic_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), FALSE);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -394,12 +472,12 @@ pygtk_tree_model_iter_has_child(GtkTreeModel *tree_model, GtkTreeIter *iter)
 }
 
 static gint
-pygtk_tree_model_iter_n_children(GtkTreeModel *tree_model, GtkTreeIter *iter)
+pygtk_generic_tree_model_iter_n_children(GtkTreeModel *tree_model, GtkTreeIter *iter)
 {
     PyObject *self, *py_ret;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), FALSE);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -422,13 +500,13 @@ pygtk_tree_model_iter_n_children(GtkTreeModel *tree_model, GtkTreeIter *iter)
 }
 
 static gboolean
-pygtk_tree_model_iter_nth_child(GtkTreeModel *tree_model, GtkTreeIter  *iter,
+pygtk_generic_tree_model_iter_nth_child(GtkTreeModel *tree_model, GtkTreeIter  *iter,
 				GtkTreeIter  *parent, gint n)
 {
     PyObject *self, *py_ret, *py_parent = Py_None;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), FALSE);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -436,13 +514,17 @@ pygtk_tree_model_iter_nth_child(GtkTreeModel *tree_model, GtkTreeIter  *iter,
 #ifdef DEBUG_TREE_MODEL
     g_message("iter_nth_child(%p, %p, %d)", iter, parent, n);
 #endif
-    if (parent) py_parent = (PyObject *)parent->user_data;
+    if (parent && parent->user_data != NULL)
+	py_parent = (PyObject *)parent->user_data;
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_nth_child",
 				 "(Oi)", py_parent, n);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
 	    iter->user_data = py_ret;
+	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
+		Py_DECREF((PyObject *)iter->user_data);
+	    }
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
@@ -458,13 +540,13 @@ pygtk_tree_model_iter_nth_child(GtkTreeModel *tree_model, GtkTreeIter  *iter,
 }
 
 static gboolean
-pygtk_tree_model_iter_parent(GtkTreeModel *tree_model, GtkTreeIter *iter,
+pygtk_generic_tree_model_iter_parent(GtkTreeModel *tree_model, GtkTreeIter *iter,
 			     GtkTreeIter *child)
 {
     PyObject *self, *py_ret, *py_child = Py_None;
 
     g_return_val_if_fail(tree_model != NULL, FALSE);
-    g_return_val_if_fail(PYGTK_IS_TREE_MODEL(tree_model), FALSE);
+    g_return_val_if_fail(PYGTK_IS_GENERIC_TREE_MODEL(tree_model), FALSE);
     g_return_val_if_fail(iter != NULL, FALSE);
     /* this call finds the wrapper for this GObject */
     self = pygobject_new((GObject *)tree_model);
@@ -472,13 +554,17 @@ pygtk_tree_model_iter_parent(GtkTreeModel *tree_model, GtkTreeIter *iter,
 #ifdef DEBUG_TREE_MODEL
     g_message("iter_parent(%p, %p)", iter, child);
 #endif
-    if (child) py_child = (PyObject *)child->user_data;
+    if (child && child->user_data != NULL)
+	py_child = (PyObject *)child->user_data;
     py_ret = PyObject_CallMethod(self, METHOD_PREFIX "iter_parent",
 				 "(O)", py_child);
     if (py_ret) {
 	if (py_ret != Py_None) {
 	    /* XXXX handle reference counting here */
 	    iter->user_data = py_ret;
+	    if (!PYGTK_GENERIC_TREE_MODEL(tree_model)->leak_references) {
+		Py_DECREF((PyObject *)iter->user_data);
+	    }
 	    return TRUE;
 	} else {
 	    iter->user_data = NULL;
