@@ -282,6 +282,21 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
     PyObject *tmp;
 
     switch (G_TYPE_FUNDAMENTAL(G_VALUE_TYPE(value))) {
+    case G_TYPE_INTERFACE:
+	/* we only handle interface types that have a GObject prereq */
+	if (g_type_is_a(G_VALUE_TYPE(value), G_TYPE_OBJECT)) {
+	    if (!PyObject_TypeCheck(obj, &PyGObject_Type)) {
+		return -1;
+	    }
+	    if (!G_TYPE_CHECK_INSTANCE_TYPE(pygobject_get(obj),
+					    G_VALUE_TYPE(value))) {
+		return -1;
+	    }
+	    g_value_set_object(value, pygobject_get(obj));
+	} else {
+	    return -1;
+	}
+	break;
     case G_TYPE_CHAR:
 	if ((tmp = PyObject_Str(obj)))
 	    g_value_set_char(value, PyString_AsString(tmp)[0]);
@@ -375,6 +390,22 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	    return -1;
 	}
 	break;
+    case G_TYPE_ENUM:
+	{
+	    gint val = 0;
+	    if (pyg_enum_get_value(G_VALUE_TYPE(value), obj, &val) < 0)
+		return -1;
+	    g_value_set_enum(value, val);
+	}
+	break;
+    case G_TYPE_FLAGS:
+	{
+	    guint val = 0;
+	    if (pyg_flags_get_value(G_VALUE_TYPE(value), obj, &val) < 0)
+		return -1;
+	    g_value_set_flags(value, val);
+	}
+	break;
     case G_TYPE_FLOAT:
 	g_value_set_float(value, PyFloat_AsDouble(obj));
 	if (PyErr_Occurred()) {
@@ -400,30 +431,11 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 	}
 	Py_DECREF(tmp);
 	break;
-    case G_TYPE_OBJECT:
-	{
-	    PyTypeObject *type = pygobject_lookup_class(G_VALUE_TYPE(value));
-	    if (!PyObject_TypeCheck(obj, type)) {
-		return -1;
-	    }
-	    g_value_set_object(value, pygobject_get(obj));
-	}
-	break;
-    case G_TYPE_ENUM:
-	{
-	    gint val = 0;
-	    if (pyg_enum_get_value(G_VALUE_TYPE(value), obj, &val) < 0)
-		return -1;
-	    g_value_set_enum(value, val);
-	}
-	break;
-    case G_TYPE_FLAGS:
-	{
-	    guint val = 0;
-	    if (pyg_flags_get_value(G_VALUE_TYPE(value), obj, &val) < 0)
-		return -1;
-	    g_value_set_flags(value, val);
-	}
+    case G_TYPE_POINTER:
+	if (PyCObject_Check(obj))
+	    g_value_set_pointer(value, PyCObject_AsVoidPtr(obj));
+	else
+	    return -1;
 	break;
     case G_TYPE_BOXED:
 	{
@@ -442,24 +454,49 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
 		return -1;
 	}
 	break;
-    case G_TYPE_POINTER:
-	if (PyCObject_Check(obj))
-	    g_value_set_pointer(value, PyCObject_AsVoidPtr(obj));
+    case G_TYPE_PARAM:
+	if (PyGParamSpec_Check(obj))
+	    g_value_set_param(value, PyCObject_AsVoidPtr(obj));
 	else
 	    return -1;
 	break;
+    case G_TYPE_OBJECT:
+	if (!PyObject_TypeCheck(obj, &PyGObject_Type)) {
+	    return -1;
+	}
+	if (!G_TYPE_CHECK_INSTANCE_TYPE(pygobject_get(obj),
+					G_VALUE_TYPE(value))) {
+	    return -1;
+	}
+	g_value_set_object(value, pygobject_get(obj));
+	break;
     default:
 	break;
     }
     return 0;
 }
 
+/**
+ * pyg_value_as_pyobject:
+ * @value: the GValue object.
+ * @copy_boxed: true if boxed values should be copied.
+ *
+ * This function creates/returns a Python wrapper object that
+ * represents the GValue passed as an argument.
+ *
+ * Returns: a PyObject representing the value.
+ */
 PyObject *
-pyg_value_as_pyobject(const GValue *value)
+pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
 {
     gchar buf[128];
 
     switch (G_TYPE_FUNDAMENTAL(G_VALUE_TYPE(value))) {
+    case G_TYPE_INTERFACE:
+	if (g_type_is_a(G_VALUE_TYPE(value), G_TYPE_OBJECT))
+	    return pygobject_new(g_value_get_object(value));
+	else
+	    break;
     case G_TYPE_CHAR:
 	{
 	    gint8 val = g_value_get_char(value);
@@ -476,12 +513,15 @@ pyg_value_as_pyobject(const GValue *value)
 	return PyInt_FromLong(g_value_get_int(value));
     case G_TYPE_UINT:
 	{
-	    gulong val = (gulong) g_value_get_uint(value);
-
-	    if (val <= G_MAXLONG)
-		return PyInt_FromLong((glong) val);
-	    else
-		return PyLong_FromUnsignedLong(val);
+	    /* in Python, the Int object is backed by a long.  If a
+	       long can hold the whole value of an unsigned int, use
+	       an Int.  Otherwise, use a Long object to avoid overflow.
+	       This matches the ULongArg behavior in codegen/argtypes.h */
+#if (G_MAXUINT <= G_MAXLONG)
+	    return PyInt_FromLong((glong) g_value_get_uint(value));
+#else
+	    return PyLong_FromUnsignedLong((gulong) g_value_get_uint(value));
+#endif
 	}
     case G_TYPE_LONG:
 	return PyInt_FromLong(g_value_get_long(value));
@@ -512,6 +552,10 @@ pyg_value_as_pyobject(const GValue *value)
 	    else
 		return PyLong_FromUnsignedLongLong(val);
 	}
+    case G_TYPE_ENUM:
+	return PyInt_FromLong(g_value_get_enum(value));
+    case G_TYPE_FLAGS:
+	return PyInt_FromLong(g_value_get_flags(value));
     case G_TYPE_FLOAT:
 	return PyFloat_FromDouble(g_value_get_float(value));
     case G_TYPE_DOUBLE:
@@ -525,28 +569,37 @@ pyg_value_as_pyobject(const GValue *value)
 	    Py_INCREF(Py_None);
 	    return Py_None;
 	}
-    case G_TYPE_OBJECT:
-	return pygobject_new(g_value_get_object(value));
-    case G_TYPE_ENUM:
-	return PyInt_FromLong(g_value_get_enum(value));
-    case G_TYPE_FLAGS:
-	return PyInt_FromLong(g_value_get_flags(value));
+    case G_TYPE_POINTER:
+	return PyCObject_FromVoidPtr(g_value_get_pointer(value), NULL);
     case G_TYPE_BOXED:
 	{
 	    PyGBoxedMarshal *bm;
 
-	    if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT))
-		return (PyObject *)g_value_dup_boxed(value);
-
+	    if (G_VALUE_HOLDS(value, PY_TYPE_OBJECT)) {
+		PyObject *ret = (PyObject *)g_value_dup_boxed(value);
+		if (ret == NULL) {
+		    Py_INCREF(Py_None);
+		    return Py_None;
+		}
+		return ret;
+	    }
+	    
 	    bm = pyg_boxed_lookup(G_VALUE_TYPE(value));
-	    if (bm)
+	    if (bm) {
 		return bm->fromvalue(value);
-	    else
-		return pyg_boxed_new(G_VALUE_TYPE(value),
-				     g_value_get_boxed(value), TRUE, TRUE);
+	    } else {
+		if (copy_boxed)
+		    return pyg_boxed_new(G_VALUE_TYPE(value),
+					 g_value_get_boxed(value), TRUE, TRUE);
+		else
+		    return pyg_boxed_new(G_VALUE_TYPE(value),
+					 g_value_get_boxed(value),FALSE,FALSE);
+	    }
 	}
-    case G_TYPE_POINTER:
-	return PyCObject_FromVoidPtr(g_value_get_pointer(value), NULL);
+    case G_TYPE_PARAM:
+	return pyg_param_spec_new(g_value_get_param(value));
+    case G_TYPE_OBJECT:
+	return pygobject_new(g_value_get_object(value));
     default:
 	break;
     }
@@ -558,23 +611,18 @@ pyg_value_as_pyobject(const GValue *value)
 
 /* -------------- PyGClosure ----------------- */
 
-typedef struct _PyGClosure PyGClosure;
-struct _PyGClosure {
-    GClosure closure;
-    PyObject *callback;
-    PyObject *extra_args; /* tuple of extra args to pass to callback */
-    PyObject *swap_data; /* other object for gtk_signal_connect_object */
-};
-
 static void
-pyg_closure_destroy(gpointer data, GClosure *closure)
+pyg_closure_invalidate(gpointer data, GClosure *closure)
 {
     PyGClosure *pc = (PyGClosure *)closure;
 
     pyg_block_threads();
-    Py_DECREF(pc->callback);
+    Py_XDECREF(pc->callback);
     Py_XDECREF(pc->extra_args);
     Py_XDECREF(pc->swap_data);
+    pc->callback = NULL;
+    pc->extra_args = NULL;
+    pc->swap_data = NULL;
     pyg_unblock_threads();
 }
 
@@ -601,7 +649,7 @@ pyg_closure_marshal(GClosure *closure,
 	    Py_INCREF(pc->swap_data);
 	    PyTuple_SetItem(params, 0, pc->swap_data);
 	} else {
-	    PyObject *item = pyg_value_as_pyobject(&param_values[i]);
+	    PyObject *item = pyg_value_as_pyobject(&param_values[i], FALSE);
 
 	    /* error condition */
 	    if (!item) {
@@ -622,11 +670,13 @@ pyg_closure_marshal(GClosure *closure,
     if (ret == NULL) {
 	PyErr_Print();
 	PyErr_Clear();
+	Py_DECREF(params);
 	pyg_unblock_threads();
 	return;
     }
     if (return_value)
 	pyg_value_from_pyobject(return_value, ret);
+    Py_DECREF(params);
     Py_DECREF(ret);
     pyg_unblock_threads();
 }
@@ -638,7 +688,7 @@ pyg_closure_new(PyObject *callback, PyObject *extra_args, PyObject *swap_data)
 
     g_return_val_if_fail(callback != NULL, NULL);
     closure = g_closure_new_simple(sizeof(PyGClosure), NULL);
-    g_closure_add_finalize_notifier(closure, NULL, pyg_closure_destroy);
+    g_closure_add_invalidate_notifier(closure, NULL, pyg_closure_invalidate);
     g_closure_set_marshal(closure, pyg_closure_marshal);
     Py_INCREF(callback);
     ((PyGClosure *)closure)->callback = callback;
@@ -720,7 +770,7 @@ pyg_signal_class_closure_marshal(GClosure *closure,
     /* construct Python tuple for the parameter values */
     params = PyTuple_New(n_param_values - 1);
     for (i = 1; i < n_param_values; i++) {
-	PyObject *item = pyg_value_as_pyobject(&param_values[i]);
+	PyObject *item = pyg_value_as_pyobject(&param_values[i], TRUE);
 
 	/* error condition */
 	if (!item) {
@@ -736,10 +786,12 @@ pyg_signal_class_closure_marshal(GClosure *closure,
 	PyErr_Print();
 	PyErr_Clear();
 	Py_DECREF(method);
+	Py_DECREF(params);
 	pyg_unblock_threads();
 	return;
     }
     Py_DECREF(method);
+    Py_DECREF(params);
     if (return_value)
 	pyg_value_from_pyobject(return_value, ret);
     Py_DECREF(ret);
